PRAGMA foreign_keys = ON;
BEGIN TRANSACTION;

-- 创建执行控制数据表
CREATE TABLE IF NOT EXISTS exectl_list (
    ID INTEGER PRIMARY KEY AUTOINCREMENT,
    fpath TEXT NOT NULL UNIQUE,
    hash TEXT NOT NULL,  -- 文件内容的SM3值
    ftype INTEGER NOT NULL, -- 文件类型 1:program, 2:library, 3:kmodule, 4:script, 5:other
    dtype INTEGER NOT NULL DEFAULT 0, -- 执行控制数据类型：0 初始化时添加的数据，1 初始化后添加的数据
    status INTEGER NOT NULL DEFAULT 0 -- 执行控制保护状态：0 正常，1 已篡改
);

-- 白名单
--创建记录视图
DROP VIEW IF EXISTS exectl_whitelist_view;
CREATE VIEW IF NOT EXISTS exectl_whitelist_view AS
SELECT ID, fpath, hash, ftype, dtype, status FROM exectl_list;

--记录表插入不存在的记录
DROP TRIGGER IF EXISTS exectl_whitelist_insert_not_exists;
CREATE TRIGGER IF NOT EXISTS exectl_whitelist_insert_not_exists
INSTEAD OF INSERT ON exectl_whitelist_view WHEN NOT EXISTS(select 1 FROM exectl_list WHERE fpath = NEW.fpath)
BEGIN
	INSERT INTO exectl_list(fpath, hash, ftype, dtype, status) values(NEW.fpath, NEW.hash, NEW.ftype, NEW.dtype, NEW.status);
END;

--记录表插入存在的记录
DROP TRIGGER IF EXISTS exectl_whitelist_insert_exists;
CREATE TRIGGER IF NOT EXISTS exectl_whitelist_insert_exists
INSTEAD OF INSERT ON exectl_whitelist_view WHEN EXISTS(select 1 FROM exectl_list WHERE fpath = NEW.fpath)
BEGIN
	UPDATE exectl_list SET hash = NEW.hash, ftype = NEW.ftype, dtype = NEW.dtype, status = NEW.status WHERE fpath = NEW.fpath;
END;

--记录表删除记录
DROP TRIGGER IF EXISTS exectl_whitelist_delete;
CREATE TRIGGER IF NOT EXISTS exectl_whitelist_delete
INSTEAD OF DELETE ON exectl_whitelist_view
BEGIN
	DELETE FROM exectl_list WHERE fpath = OLD.fpath;
END;

--记录表更新记录
DROP TRIGGER IF EXISTS exectl_whitelist_update;
CREATE TRIGGER IF NOT EXISTS exectl_whitelist_update
INSTEAD OF UPDATE ON exectl_whitelist_view WHEN EXISTS(select 1 FROM exectl_list WHERE fpath = NEW.fpath)
BEGIN
	UPDATE exectl_list SET hash = NEW.hash, ftype = NEW.ftype, dtype = NEW.dtype, status = NEW.status WHERE fpath = NEW.fpath;
END;

COMMIT;
